/// A macro that will define a binary operation for
/// a structure and its components.
macro_rules! binary_operator_impl
{
   ($traitName: ident :: $funcName: ident,
    $generic_type: path $(| $generic_types2: path)*,
    $structName: ident {$($field: ident),+}) =>
   {
      impl<T> $traitName<T> for $structName<T>
         where T: $generic_type $(+ $generic_types2)*
      {
         type Output = $structName<T>;

         fn $funcName(self, val: T) -> $structName<T>
         {
            $structName::new($(self.$field.$funcName(val)),+)
         }
      }

      impl<'a, T> $traitName<T> for &'a $structName<T>
         where T: $generic_type $(+ $generic_types2)*
      {
         type Output = $structName<T>;

         fn $funcName(self, val: T) -> $structName<T>
         {
            $structName::new($(self.$field.$funcName(val)),+)
         }
      }

      impl<'a, T> $traitName<&'a T> for $structName<T>
         where T: $generic_type $(+ $generic_types2)*
      {
         type Output = $structName<T>;

         fn $funcName(self, val: &'a T) -> $structName<T>
         {
            $structName::new($(self.$field.$funcName(*val)),+)
         }
      }

      impl<'a, 'b, T> $traitName<&'b T> for &'a $structName<T>
         where T: $generic_type $(+ $generic_types2)*
      {
         type Output = $structName<T>;

         fn $funcName(self, val: &'b T) -> $structName<T>
         {
            $structName::new($(self.$field.$funcName(*val)),+)
         }
      }

      impl<T> $traitName<$structName<T>> for $structName<T>
         where T: $generic_type $(+ $generic_types2)*
      {
         type Output = $structName<T>;

         fn $funcName(self, other: $structName<T>) -> $structName<T>
         {
            $structName::new($(self.$field.$funcName(other.$field)),+)
         }
      }

      impl<'a, T> $traitName<$structName<T>> for &'a $structName<T>
         where T: $generic_type $(+ $generic_types2)*
      {
         type Output = $structName<T>;

         fn $funcName(self, other: $structName<T>) -> $structName<T>
         {
            $structName::new($(self.$field.$funcName(other.$field)),+)
         }
      }

      impl<'a, T> $traitName<&'a $structName<T>> for $structName<T>
         where T: $generic_type $(+ $generic_types2)*
      {
         type Output = $structName<T>;

         fn $funcName(self, other: &'a $structName<T>) -> $structName<T>
         {
            $structName::new($(self.$field.$funcName(other.$field)),+)
         }
      }

      impl<'a, 'b, T> $traitName<&'a $structName<T>> for &'b $structName<T>
         where T: $generic_type $(+ $generic_types2)*
      {
         type Output = $structName<T>;

         fn $funcName(self, other: &'a $structName<T>) -> $structName<T>
         {
            $structName::new($(self.$field.$funcName(other.$field)),+)
         }
      }
   }
}

/// A macro that will define an assignment binary operation for
/// a structure and its components.
macro_rules! binary_operator_assign_impl
{
   ($traitName: ident :: $funcName: ident,
    $generic_type: path $(| $generic_types2: path)*,
    $structName: ident {$($field: ident),+}) =>
   {
      impl<T> $traitName<$structName<T>> for $structName<T>
         where T: $generic_type $(+ $generic_types2)*
      {
         fn $funcName(&mut self, rhs: $structName<T>)
         {
            $(self.$field.$funcName(rhs.$field);)+
         }
      }

      impl<T> $traitName<T> for $structName<T>
         where T: $generic_type $(+ $generic_types2)*
      {
         fn $funcName(&mut self, rhs: T)
         {
            $(self.$field.$funcName(rhs);)+
         }
      }

      impl<'a, T> $traitName<&'a $structName<T>> for $structName<T>
         where T: $generic_type $(+ $generic_types2)*
      {
         fn $funcName(&mut self, rhs: &'a $structName<T>)
         {
            $(self.$field.$funcName(rhs.$field);)+
         }
      }

      impl<'a, T> $traitName<&'a T> for $structName<T>
         where T: $generic_type $(+ $generic_types2)*
      {
         fn $funcName(&mut self, rhs: &'a T)
         {
            $(self.$field.$funcName(*rhs);)+
         }
      }
   }
}
